string name="Liberty";
string description="Distortion";

array<string> inputParametersNames={"Type","Input Gain","Modulate","Output Gain"};
array<double> inputParameters(inputParametersNames.length);

double lastLeftBuff=0;
double lastRightBuff=0;
double lastLeftSamp=0;
double lastRightSamp=0;
double modulate = 0;
double type = 0;
double horizon = 0;
double stay = 0;
double val = 0;
double reduction = 1;
double runnerL = 0;
double runnerR = 0;
double input_gain = 0;

void updateInputParameters()
{
	type = inputParameters[0];
	input_gain = inputParameters[1] * 4.0;
	modulate = inputParameters[2];
	reduction = inputParameters[3] * 2.0;
}

void processSample(array<double>& ioSample)
{
	for(uint channel=0;channel<audioInputsCount;channel++)
	{
		ioSample[channel] *= input_gain;
		if (channel > 1) ioSample[channel] = 0.0;
		else
		switch (int(floor(type * 13)))
		{
			// Long Saw
			case 0:
				horizon = ioSample[channel];
				if (channel == 0)
				{
					if (ioSample[channel] > lastLeftSamp)
					{
						lastLeftBuff = ioSample[channel];
					}
					lastLeftSamp = ioSample[channel];
					ioSample[channel] = lastLeftBuff;
				}
				if (channel == 1)
				{
					if (ioSample[channel] > lastRightSamp)
					{
						lastRightBuff = ioSample[channel];
					}
					lastRightSamp = ioSample[channel];
					ioSample[channel] = lastRightBuff;
				}
				ioSample[channel] = ioSample[channel] * modulate + horizon * (1- modulate);
			break;

			//HighSaw
			case 1:
				horizon = ioSample[channel];
				if (channel == 0)
				{
					lastLeftBuff = ioSample[channel];
					if (ioSample[channel] > lastLeftSamp)
					{
						lastLeftBuff = ioSample[channel] * -1.0;
					}
					lastLeftSamp = ioSample[channel];
					ioSample[channel] = lastLeftBuff;
				}
				if (channel == 1)
				{
					lastRightBuff = ioSample[channel];
					if (ioSample[channel] > lastRightSamp)
					{
						lastRightBuff = ioSample[channel] * -1.0;
					}
					lastRightSamp = ioSample[channel];
					ioSample[channel] = lastRightBuff;
				}
				ioSample[channel] = ioSample[channel] * modulate + horizon * (1- modulate);
			break;

			// zero threshold
			case 2:
				if (channel == 0 || channel == 1)
				{
					if (abs(ioSample[channel]) - modulate < 0) ioSample[channel] = 0;
				}
			break;

			// horizon
			case 3:
				if (channel == 0 || channel == 1)
				{
					horizon = abs(ioSample[channel]) - (modulate * 1);
					//if (horizon < 0) horizon = 0;
					if (horizon < 0) horizon *= -1;
					if (ioSample[channel] >= 0) ioSample[channel] = horizon;
					else ioSample[channel] = -1 * horizon;
				}

			break;

			// wave fold mod
			case 4:
				if (channel == 0 || channel == 1)
				{
					ioSample[channel] = (ioSample[channel] * ((modulate*25)+1)) % 1;
				}
			break;

			case 5:
				if (channel == 0 || channel == 1)
				{
					if (ioSample[channel] >= 0) ioSample[channel] = ((ioSample[channel] * ((modulate*25)+1)) % 1)*2-1;
					else ioSample[channel] = ioSample[channel] = (ioSample[channel] * ((modulate*25)+1)) % 1*2+1;
				}
			break;


			// wave fold
			case 6:
				if (channel == 0 || channel == 1)
				{
					ioSample[channel] *= (modulate*50+1);
					val = ioSample[channel];
					if (val > 1)
					{
						stay = (val - 1) * 0.5;
						if (stay % 2 >= 1)
							ioSample[channel] = -1 + (val - (floor(stay)*2+1));
						else
							ioSample[channel] = 1 -  (val - (floor(stay)*2+1));
					}
					if (val < -1)
					{
						val = abs(val);
						stay = (val - 1) * 0.5;
						if (stay % 2 >= 1)
							ioSample[channel] = 1 - (val - (floor(stay)*2+1));
						else
							ioSample[channel] = -1 +  (val - (floor(stay)*2+1));
					}
					
				}
			break;

			// sign method
			case 7:
				horizon = ioSample[channel];
				if (channel == 0 || channel == 1)
				{
					if (ioSample[channel] >= 0) ioSample[channel] = 1;
					else ioSample[channel] = -1;
				}
				ioSample[channel] = ioSample[channel] * modulate + horizon * (1- modulate);
			break;

			// sigmoid
			case 8:
				if (channel == 0 || channel == 1)
				{
					ioSample[channel] = ioSample[channel] / (abs(ioSample[channel]) + (( ((1-modulate) * (1-modulate)) + 0.0001) * 2));
				}
			break;

			// amplitude to sine
			case 9:
				if (channel == 0 || channel == 1)
				{
					ioSample[channel] = sin(ioSample[channel] * 3.14159265 * 2 * (modulate + 0.0002) * 300);
				}
			break;

			// amplitude to sine
			case 10:
				if (channel == 0 || channel == 1)
				{
					ioSample[channel] = sin(abs(ioSample[channel] * 3.14159265 * 2) * (modulate+0.0001) * 600);
				}
			break;

			// add one that restores the wave direction
			case 11:
				if (channel == 0 || channel == 1)
				{
					
					if (ioSample[channel] >= 0) ioSample[channel] = sin(abs(ioSample[channel] * 3.14159265 * 2) * (modulate+0.0001) * 600);
					else ioSample[channel] = ioSample[channel] = -1 * sin(abs(ioSample[channel] * 3.14159265 * 2) * (modulate+0.0001) * 600);
				}
			break;

			// amplitude to sine running speed
			case 12:
				if (channel == 0)
				{
					runnerL = (runnerL + (ioSample[channel] * (modulate + 0.005))) % 1.0;
					ioSample[channel] = sin(runnerL * 3.14159265 * 2);	
				}
				if (channel == 1)
				{
					runnerR = (runnerR + (ioSample[channel] * (modulate + 0.005))) % 1.0;
					ioSample[channel] = sin(runnerR * 3.14159265 * 2);	
				}
			break;
		}
		ioSample[channel] *= reduction;
	}	
}

